@@ -116,6 +116,15 @@ _gst_tag_initialize (void)
       G_TYPE_STRING,
       _("album sortname"),
       _("album containing this data for sorting purposes"), NULL);
+  gst_tag_register (GST_TAG_ALBUM_ARTIST, GST_TAG_FLAG_META,
+      G_TYPE_STRING,
+      _("album artist"),
+      _("The artist of the entire album, as it should be displayed"),
+      gst_tag_merge_strings_with_comma);
+  gst_tag_register (GST_TAG_ALBUM_ARTIST_SORTNAME, GST_TAG_FLAG_META,
+      G_TYPE_STRING,
+      _("album artist sortname"),
+      _("The artist of the entire album, as it should be sorted"), NULL);
   gst_tag_register (GST_TAG_DATE, GST_TAG_FLAG_META, GST_TYPE_DATE,
       _("date"), _("date the data was created (as a GDate structure)"), NULL);
   gst_tag_register (GST_TAG_GENRE, GST_TAG_FLAG_META,
